==> jacoco-maven-plugin-0.5.5.201112152213-sources.jar: org/jacoco/maven/HelpMojo.java <==
package org.jacoco.maven;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * Display help information on jacoco-maven-plugin.<br/> Call <pre>  mvn jacoco:help -Ddetail=true -Dgoal=&lt;goal-name&gt;</pre> to display parameter details.
 *
 * @version generated on Fri Dec 16 01:40:54 MSK 2011
 * @author org.apache.maven.tools.plugin.generator.PluginHelpGenerator (version 2.8)
 * @goal help
 * @requiresProject false
 * @threadSafe
 */
public class HelpMojo
    extends AbstractMojo
{
    /**
     * If <code>true</code>, display all settable properties for each goal.
     * 
     * @parameter expression="${detail}" default-value="false"
     */
    private boolean detail;

    /**
     * The name of the goal for which to show help. If unspecified, all goals will be displayed.
     * 
     * @parameter expression="${goal}"
     */
    private java.lang.String goal;

    /**
     * The maximum length of a display line, should be positive.
     * 
     * @parameter expression="${lineLength}" default-value="80"
     */
    private int lineLength;

    /**
     * The number of spaces per indentation level, should be positive.
     * 
     * @parameter expression="${indentSize}" default-value="2"
     */
    private int indentSize;


    /** {@inheritDoc} */
    public void execute()
        throws MojoExecutionException
    {
        if ( lineLength <= 0 )
        {
            getLog().warn( "The parameter 'lineLength' should be positive, using '80' as default." );
            lineLength = 80;
        }
        if ( indentSize <= 0 )
        {
            getLog().warn( "The parameter 'indentSize' should be positive, using '2' as default." );
            indentSize = 2;
        }

        StringBuffer sb = new StringBuffer();

        append( sb, "org.jacoco:jacoco-maven-plugin:0.5.5.201112152213", 0 );
        append( sb, "", 0 );

        append( sb, "JaCoCo :: Maven Plugin", 0 );
        append( sb, "The JaCoCo Maven Plugin provides the JaCoCo runtime agent to your tests and allows basic report creation.", 1 );
        append( sb, "", 0 );

        if ( goal == null || goal.length() <= 0 )
        {
            append( sb, "This plugin has 3 goals:", 0 );
            append( sb, "", 0 );
        }

        if ( goal == null || goal.length() <= 0 || "help".equals( goal ) )
        {
            append( sb, "jacoco:help", 0 );
            append( sb, "Display help information on jacoco-maven-plugin.\nCall\n\u00a0\u00a0mvn\u00a0jacoco:help\u00a0-Ddetail=true\u00a0-Dgoal=<goal-name>\nto display parameter details.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "detail (Default: false)", 2 );
                append( sb, "If true, display all settable properties for each goal.", 3 );
                append( sb, "Expression: ${detail}", 3 );
                append( sb, "", 0 );

                append( sb, "goal", 2 );
                append( sb, "The name of the goal for which to show help. If unspecified, all goals will be displayed.", 3 );
                append( sb, "Expression: ${goal}", 3 );
                append( sb, "", 0 );

                append( sb, "indentSize (Default: 2)", 2 );
                append( sb, "The number of spaces per indentation level, should be positive.", 3 );
                append( sb, "Expression: ${indentSize}", 3 );
                append( sb, "", 0 );

                append( sb, "lineLength (Default: 80)", 2 );
                append( sb, "The maximum length of a display line, should be positive.", 3 );
                append( sb, "Expression: ${lineLength}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "prepare-agent".equals( goal ) )
        {
            append( sb, "jacoco:prepare-agent", 0 );
            append( sb, "Prepares a property pointing to the JaCoCo runtime agent that can be passed as a VM argument to the application under test. Depending on the project packaging type by default a property with the following name is set:\n-\ttycho.testArgLine for packaging type eclipse-test-plugin and\n-\targLine otherwise.\nResulting coverage information is collected during execution and by default written to a file when the process terminates.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "address", 2 );
                append( sb, "IP address or hostname to bind to when the output method is tcpserver or connect to when the output method is tcpclient. In tcpserver mode the value \'*\' causes the agent to accept connections on any local address.", 3 );
                append( sb, "Expression: ${jacoco.address}", 3 );
                append( sb, "", 0 );

                append( sb, "append", 2 );
                append( sb, "If set to true and the execution data file already exists, coverage data is appended to the existing file. If set to false, an existing execution data file will be replaced.", 3 );
                append( sb, "Expression: ${jacoco.append}", 3 );
                append( sb, "", 0 );

                append( sb, "destFile (Default: ${project.build.directory}/jacoco.exec)", 2 );
                append( sb, "Path to the output file for execution data.", 3 );
                append( sb, "Expression: ${jacoco.destFile}", 3 );
                append( sb, "", 0 );

                append( sb, "dumpOnExit", 2 );
                append( sb, "If set to true coverage data will be written on VM shutdown.", 3 );
                append( sb, "Expression: ${jacoco.dumpOnExit}", 3 );
                append( sb, "", 0 );

                append( sb, "exclClassLoaders", 2 );
                append( sb, "A list of class loader names, that should be excluded from execution analysis. The list entries are separated by a colon (:) and may use wildcard characters (* and ?). This option might be required in case of special frameworks that conflict with JaCoCo code instrumentation, in particular class loaders that do not have access to the Java runtime classes.", 3 );
                append( sb, "Expression: ${jacoco.exclClassLoaders}", 3 );
                append( sb, "", 0 );

                append( sb, "excludes", 2 );
                append( sb, "A list of class files to exclude from instrumentation/analysis/reports. May use wildcard characters (* and ?).", 3 );
                append( sb, "Expression: ${jacoco.excludes}", 3 );
                append( sb, "", 0 );

                append( sb, "includes", 2 );
                append( sb, "A list of class files to include in instrumentation/analysis/reports. May use wildcard characters (* and ?). When not specified - everything will be included.", 3 );
                append( sb, "Expression: ${jacoco.includes}", 3 );
                append( sb, "", 0 );

                append( sb, "output", 2 );
                append( sb, "Output method to use for writing coverage data. Valid options are:\n-\tfile: At VM termination execution data is written to the file specified in the destfile.\n-\ttcpserver: The agent listens for incoming connections on the TCP port specified by the address and port. Execution data is written to this TCP connection.\n-\ttcpclient: At startup the agent connects to the TCP port specified by the address and port. Execution data is written to this TCP connection.\n-\tmbean: The agent registers an JMX MBean under the name org.jacoco:type=Runtime.\n", 3 );
                append( sb, "Expression: ${jacoco.output}", 3 );
                append( sb, "", 0 );

                append( sb, "port", 2 );
                append( sb, "Port to bind to when the output method is tcpserver or connect to when the output method is tcpclient. In tcpserver mode the port must be available, which means that if multiple JaCoCo agents should run on the same machine, different ports have to be specified.", 3 );
                append( sb, "Expression: ${jacoco.port}", 3 );
                append( sb, "", 0 );

                append( sb, "propertyName", 2 );
                append( sb, "Allows to specify property which will contains settings for JaCoCo Agent. If not specified, then \'argLine\' would be used for \'jar\' packaging and \'tycho.testArgLine\' for \'eclipse-test-plugin\'.", 3 );
                append( sb, "Expression: ${jacoco.propertyName}", 3 );
                append( sb, "", 0 );

                append( sb, "sessionId", 2 );
                append( sb, "A session identifier that is written with the execution data. Without this parameter a random identifier is created by the agent.", 3 );
                append( sb, "Expression: ${jacoco.sessionId}", 3 );
                append( sb, "", 0 );

                append( sb, "skip (Default: false)", 2 );
                append( sb, "Flag used to suppress execution.", 3 );
                append( sb, "Expression: ${jacoco.skip}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "report".equals( goal ) )
        {
            append( sb, "jacoco:report", 0 );
            append( sb, "Creates a code coverage report for a single project in multiple formats (HTML, XML, and CSV).", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "dataFile (Default: ${project.build.directory}/jacoco.exec)", 2 );
                append( sb, "File with execution data.", 3 );
                append( sb, "", 0 );

                append( sb, "excludes", 2 );
                append( sb, "A list of class files to exclude from instrumentation/analysis/reports. May use wildcard characters (* and ?).", 3 );
                append( sb, "Expression: ${jacoco.excludes}", 3 );
                append( sb, "", 0 );

                append( sb, "includes", 2 );
                append( sb, "A list of class files to include in instrumentation/analysis/reports. May use wildcard characters (* and ?). When not specified - everything will be included.", 3 );
                append( sb, "Expression: ${jacoco.includes}", 3 );
                append( sb, "", 0 );

                append( sb, "outputDirectory (Default: ${project.reporting.outputDirectory}/jacoco)", 2 );
                append( sb, "Output directory for the reports.", 3 );
                append( sb, "", 0 );

                append( sb, "outputEncoding (Default: UTF-8)", 2 );
                append( sb, "Encoding of the generated reports.", 3 );
                append( sb, "Expression: ${project.reporting.outputEncoding}", 3 );
                append( sb, "", 0 );

                append( sb, "skip (Default: false)", 2 );
                append( sb, "Flag used to suppress execution.", 3 );
                append( sb, "Expression: ${jacoco.skip}", 3 );
                append( sb, "", 0 );

                append( sb, "sourceEncoding (Default: UTF-8)", 2 );
                append( sb, "Encoding of the source files.", 3 );
                append( sb, "Expression: ${project.build.sourceEncoding}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( getLog().isInfoEnabled() )
        {
            getLog().info( sb.toString() );
        }
    }

    /**
     * <p>Repeat a String <code>n</code> times to form a new string.</p>
     *
     * @param str String to repeat
     * @param repeat number of times to repeat str
     * @return String with repeated String
     * @throws NegativeArraySizeException if <code>repeat < 0</code>
     * @throws NullPointerException if str is <code>null</code>
     */
    private static String repeat( String str, int repeat )
    {
        StringBuffer buffer = new StringBuffer( repeat * str.length() );

        for ( int i = 0; i < repeat; i++ )
        {
            buffer.append( str );
        }

        return buffer.toString();
    }

    /** 
     * Append a description to the buffer by respecting the indentSize and lineLength parameters.
     * <b>Note</b>: The last character is always a new line.
     * 
     * @param sb The buffer to append the description, not <code>null</code>.
     * @param description The description, not <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     */
    private void append( StringBuffer sb, String description, int indent )
    {
        for ( Iterator it = toLines( description, indent, indentSize, lineLength ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( '\n' );
        }
    }

    /** 
     * Splits the specified text into lines of convenient display length.
     * 
     * @param text The text to split into lines, must not be <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     * @return The sequence of display lines, never <code>null</code>.
     * @throws NegativeArraySizeException if <code>indent < 0</code>
     */
    private static List toLines( String text, int indent, int indentSize, int lineLength )
    {
        List lines = new ArrayList();

        String ind = repeat( "\t", indent );
        String[] plainLines = text.split( "(\r\n)|(\r)|(\n)" );
        for ( int i = 0; i < plainLines.length; i++ )
        {
            toLines( lines, ind + plainLines[i], indentSize, lineLength );
        }

        return lines;
    }

    /** 
     * Adds the specified line to the output sequence, performing line wrapping if necessary.
     * 
     * @param lines The sequence of display lines, must not be <code>null</code>.
     * @param line The line to add, must not be <code>null</code>.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     */
    private static void toLines( List lines, String line, int indentSize, int lineLength )
    {
        int lineIndent = getIndentLevel( line );
        StringBuffer buf = new StringBuffer( 256 );
        String[] tokens = line.split( " +" );
        for ( int i = 0; i < tokens.length; i++ )
        {
            String token = tokens[i];
            if ( i > 0 )
            {
                if ( buf.length() + token.length() >= lineLength )
                {
                    lines.add( buf.toString() );
                    buf.setLength( 0 );
                    buf.append( repeat( " ", lineIndent * indentSize ) );
                }
                else
                {
                    buf.append( ' ' );
                }
            }
            for ( int j = 0; j < token.length(); j++ )
            {
                char c = token.charAt( j );
                if ( c == '\t' )
                {
                    buf.append( repeat( " ", indentSize - buf.length() % indentSize ) );
                }
                else if ( c == '\u00A0' )
                {
                    buf.append( ' ' );
                }
                else
                {
                    buf.append( c );
                }
            }
        }
        lines.add( buf.toString() );
    }

    /** 
     * Gets the indentation level of the specified line.
     * 
     * @param line The line whose indentation level should be retrieved, must not be <code>null</code>.
     * @return The indentation level of the line.
     */
    private static int getIndentLevel( String line )
    {
        int level = 0;
        for ( int i = 0; i < line.length() && line.charAt( i ) == '\t'; i++ )
        {
            level++;
        }
        for ( int i = level + 1; i <= level + 4 && i < line.length(); i++ )
        {
            if ( line.charAt( i ) == '\t' )
            {
                level++;
                break;
            }
        }
        return level;
    }
}

==> jacoco-maven-plugin-0.5.6.201201232323-sources.jar: org/jacoco/maven/HelpMojo.java <==
package org.jacoco.maven;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * Display help information on jacoco-maven-plugin.<br/> Call <pre>  mvn jacoco:help -Ddetail=true -Dgoal=&lt;goal-name&gt;</pre> to display parameter details.
 *
 * @version generated on Tue Jan 24 03:50:09 MSK 2012
 * @author org.apache.maven.tools.plugin.generator.PluginHelpGenerator (version 2.8)
 * @goal help
 * @requiresProject false
 * @threadSafe
 */
public class HelpMojo
    extends AbstractMojo
{
    /**
     * If <code>true</code>, display all settable properties for each goal.
     * 
     * @parameter expression="${detail}" default-value="false"
     */
    private boolean detail;

    /**
     * The name of the goal for which to show help. If unspecified, all goals will be displayed.
     * 
     * @parameter expression="${goal}"
     */
    private java.lang.String goal;

    /**
     * The maximum length of a display line, should be positive.
     * 
     * @parameter expression="${lineLength}" default-value="80"
     */
    private int lineLength;

    /**
     * The number of spaces per indentation level, should be positive.
     * 
     * @parameter expression="${indentSize}" default-value="2"
     */
    private int indentSize;


    /** {@inheritDoc} */
    public void execute()
        throws MojoExecutionException
    {
        if ( lineLength <= 0 )
        {
            getLog().warn( "The parameter 'lineLength' should be positive, using '80' as default." );
            lineLength = 80;
        }
        if ( indentSize <= 0 )
        {
            getLog().warn( "The parameter 'indentSize' should be positive, using '2' as default." );
            indentSize = 2;
        }

        StringBuffer sb = new StringBuffer();

        append( sb, "org.jacoco:jacoco-maven-plugin:0.5.6.201201232323", 0 );
        append( sb, "", 0 );

        append( sb, "JaCoCo :: Maven Plugin", 0 );
        append( sb, "The JaCoCo Maven Plugin provides the JaCoCo runtime agent to your tests and allows basic report creation.", 1 );
        append( sb, "", 0 );

        if ( goal == null || goal.length() <= 0 )
        {
            append( sb, "This plugin has 3 goals:", 0 );
            append( sb, "", 0 );
        }

        if ( goal == null || goal.length() <= 0 || "help".equals( goal ) )
        {
            append( sb, "jacoco:help", 0 );
            append( sb, "Display help information on jacoco-maven-plugin.\nCall\n\u00a0\u00a0mvn\u00a0jacoco:help\u00a0-Ddetail=true\u00a0-Dgoal=<goal-name>\nto display parameter details.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "detail (Default: false)", 2 );
                append( sb, "If true, display all settable properties for each goal.", 3 );
                append( sb, "Expression: ${detail}", 3 );
                append( sb, "", 0 );

                append( sb, "goal", 2 );
                append( sb, "The name of the goal for which to show help. If unspecified, all goals will be displayed.", 3 );
                append( sb, "Expression: ${goal}", 3 );
                append( sb, "", 0 );

                append( sb, "indentSize (Default: 2)", 2 );
                append( sb, "The number of spaces per indentation level, should be positive.", 3 );
                append( sb, "Expression: ${indentSize}", 3 );
                append( sb, "", 0 );

                append( sb, "lineLength (Default: 80)", 2 );
                append( sb, "The maximum length of a display line, should be positive.", 3 );
                append( sb, "Expression: ${lineLength}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "prepare-agent".equals( goal ) )
        {
            append( sb, "jacoco:prepare-agent", 0 );
            append( sb, "Prepares a property pointing to the JaCoCo runtime agent that can be passed as a VM argument to the application under test. Depending on the project packaging type by default a property with the following name is set:\n-\ttycho.testArgLine for packaging type eclipse-test-plugin and\n-\targLine otherwise.\nResulting coverage information is collected during execution and by default written to a file when the process terminates.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "address", 2 );
                append( sb, "IP address or hostname to bind to when the output method is tcpserver or connect to when the output method is tcpclient. In tcpserver mode the value \'*\' causes the agent to accept connections on any local address.", 3 );
                append( sb, "Expression: ${jacoco.address}", 3 );
                append( sb, "", 0 );

                append( sb, "append", 2 );
                append( sb, "If set to true and the execution data file already exists, coverage data is appended to the existing file. If set to false, an existing execution data file will be replaced.", 3 );
                append( sb, "Expression: ${jacoco.append}", 3 );
                append( sb, "", 0 );

                append( sb, "destFile (Default: ${project.build.directory}/jacoco.exec)", 2 );
                append( sb, "Path to the output file for execution data.", 3 );
                append( sb, "Expression: ${jacoco.destFile}", 3 );
                append( sb, "", 0 );

                append( sb, "dumpOnExit", 2 );
                append( sb, "If set to true coverage data will be written on VM shutdown.", 3 );
                append( sb, "Expression: ${jacoco.dumpOnExit}", 3 );
                append( sb, "", 0 );

                append( sb, "exclClassLoaders", 2 );
                append( sb, "A list of class loader names, that should be excluded from execution analysis. The list entries are separated by a colon (:) and may use wildcard characters (* and ?). This option might be required in case of special frameworks that conflict with JaCoCo code instrumentation, in particular class loaders that do not have access to the Java runtime classes.", 3 );
                append( sb, "Expression: ${jacoco.exclClassLoaders}", 3 );
                append( sb, "", 0 );

                append( sb, "excludes", 2 );
                append( sb, "A list of class files to exclude from instrumentation/analysis/reports. May use wildcard characters (* and ?).", 3 );
                append( sb, "", 0 );

                append( sb, "includes", 2 );
                append( sb, "A list of class files to include in instrumentation/analysis/reports. May use wildcard characters (* and ?). When not specified - everything will be included.", 3 );
                append( sb, "", 0 );

                append( sb, "output", 2 );
                append( sb, "Output method to use for writing coverage data. Valid options are:\n-\tfile: At VM termination execution data is written to the file specified in the destfile.\n-\ttcpserver: The agent listens for incoming connections on the TCP port specified by the address and port. Execution data is written to this TCP connection.\n-\ttcpclient: At startup the agent connects to the TCP port specified by the address and port. Execution data is written to this TCP connection.\n-\tmbean: The agent registers an JMX MBean under the name org.jacoco:type=Runtime.\n", 3 );
                append( sb, "Expression: ${jacoco.output}", 3 );
                append( sb, "", 0 );

                append( sb, "port", 2 );
                append( sb, "Port to bind to when the output method is tcpserver or connect to when the output method is tcpclient. In tcpserver mode the port must be available, which means that if multiple JaCoCo agents should run on the same machine, different ports have to be specified.", 3 );
                append( sb, "Expression: ${jacoco.port}", 3 );
                append( sb, "", 0 );

                append( sb, "propertyName", 2 );
                append( sb, "Allows to specify property which will contains settings for JaCoCo Agent. If not specified, then \'argLine\' would be used for \'jar\' packaging and \'tycho.testArgLine\' for \'eclipse-test-plugin\'.", 3 );
                append( sb, "Expression: ${jacoco.propertyName}", 3 );
                append( sb, "", 0 );

                append( sb, "sessionId", 2 );
                append( sb, "A session identifier that is written with the execution data. Without this parameter a random identifier is created by the agent.", 3 );
                append( sb, "Expression: ${jacoco.sessionId}", 3 );
                append( sb, "", 0 );

                append( sb, "skip (Default: false)", 2 );
                append( sb, "Flag used to suppress execution.", 3 );
                append( sb, "Expression: ${jacoco.skip}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "report".equals( goal ) )
        {
            append( sb, "jacoco:report", 0 );
            append( sb, "Creates a code coverage report for a single project in multiple formats (HTML, XML, and CSV).", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "dataFile (Default: ${project.build.directory}/jacoco.exec)", 2 );
                append( sb, "File with execution data.", 3 );
                append( sb, "", 0 );

                append( sb, "excludes", 2 );
                append( sb, "A list of class files to exclude from instrumentation/analysis/reports. May use wildcard characters (* and ?).", 3 );
                append( sb, "", 0 );

                append( sb, "includes", 2 );
                append( sb, "A list of class files to include in instrumentation/analysis/reports. May use wildcard characters (* and ?). When not specified - everything will be included.", 3 );
                append( sb, "", 0 );

                append( sb, "outputDirectory (Default: ${project.reporting.outputDirectory}/jacoco)", 2 );
                append( sb, "Output directory for the reports. Note that this parameter is only relevant if the goal is run from the command line or from the default build lifecycle. If the goal is run indirectly as part of a site generation, the output directory configured in the Maven Site Plugin is used instead.", 3 );
                append( sb, "", 0 );

                append( sb, "outputEncoding (Default: UTF-8)", 2 );
                append( sb, "Encoding of the generated reports.", 3 );
                append( sb, "Expression: ${project.reporting.outputEncoding}", 3 );
                append( sb, "", 0 );

                append( sb, "skip (Default: false)", 2 );
                append( sb, "Flag used to suppress execution.", 3 );
                append( sb, "Expression: ${jacoco.skip}", 3 );
                append( sb, "", 0 );

                append( sb, "sourceEncoding (Default: UTF-8)", 2 );
                append( sb, "Encoding of the source files.", 3 );
                append( sb, "Expression: ${project.build.sourceEncoding}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( getLog().isInfoEnabled() )
        {
            getLog().info( sb.toString() );
        }
    }

    /**
     * <p>Repeat a String <code>n</code> times to form a new string.</p>
     *
     * @param str String to repeat
     * @param repeat number of times to repeat str
     * @return String with repeated String
     * @throws NegativeArraySizeException if <code>repeat < 0</code>
     * @throws NullPointerException if str is <code>null</code>
     */
    private static String repeat( String str, int repeat )
    {
        StringBuffer buffer = new StringBuffer( repeat * str.length() );

        for ( int i = 0; i < repeat; i++ )
        {
            buffer.append( str );
        }

        return buffer.toString();
    }

    /** 
     * Append a description to the buffer by respecting the indentSize and lineLength parameters.
     * <b>Note</b>: The last character is always a new line.
     * 
     * @param sb The buffer to append the description, not <code>null</code>.
     * @param description The description, not <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     */
    private void append( StringBuffer sb, String description, int indent )
    {
        for ( Iterator it = toLines( description, indent, indentSize, lineLength ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( '\n' );
        }
    }

    /** 
     * Splits the specified text into lines of convenient display length.
     * 
     * @param text The text to split into lines, must not be <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     * @return The sequence of display lines, never <code>null</code>.
     * @throws NegativeArraySizeException if <code>indent < 0</code>
     */
    private static List toLines( String text, int indent, int indentSize, int lineLength )
    {
        List lines = new ArrayList();

        String ind = repeat( "\t", indent );
        String[] plainLines = text.split( "(\r\n)|(\r)|(\n)" );
        for ( int i = 0; i < plainLines.length; i++ )
        {
            toLines( lines, ind + plainLines[i], indentSize, lineLength );
        }

        return lines;
    }

    /** 
     * Adds the specified line to the output sequence, performing line wrapping if necessary.
     * 
     * @param lines The sequence of display lines, must not be <code>null</code>.
     * @param line The line to add, must not be <code>null</code>.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     */
    private static void toLines( List lines, String line, int indentSize, int lineLength )
    {
        int lineIndent = getIndentLevel( line );
        StringBuffer buf = new StringBuffer( 256 );
        String[] tokens = line.split( " +" );
        for ( int i = 0; i < tokens.length; i++ )
        {
            String token = tokens[i];
            if ( i > 0 )
            {
                if ( buf.length() + token.length() >= lineLength )
                {
                    lines.add( buf.toString() );
                    buf.setLength( 0 );
                    buf.append( repeat( " ", lineIndent * indentSize ) );
                }
                else
                {
                    buf.append( ' ' );
                }
            }
            for ( int j = 0; j < token.length(); j++ )
            {
                char c = token.charAt( j );
                if ( c == '\t' )
                {
                    buf.append( repeat( " ", indentSize - buf.length() % indentSize ) );
                }
                else if ( c == '\u00A0' )
                {
                    buf.append( ' ' );
                }
                else
                {
                    buf.append( c );
                }
            }
        }
        lines.add( buf.toString() );
    }

    /** 
     * Gets the indentation level of the specified line.
     * 
     * @param line The line whose indentation level should be retrieved, must not be <code>null</code>.
     * @return The indentation level of the line.
     */
    private static int getIndentLevel( String line )
    {
        int level = 0;
        for ( int i = 0; i < line.length() && line.charAt( i ) == '\t'; i++ )
        {
            level++;
        }
        for ( int i = level + 1; i <= level + 4 && i < line.length(); i++ )
        {
            if ( line.charAt( i ) == '\t' )
            {
                level++;
                break;
            }
        }
        return level;
    }
}
